/*

######################################################
###        simple LS algorithms for the SMWC       ###
######################################################

      Version: 0.9
      File:    options-opag.c
      Author:  Thomas Stuetzle
      Purpose: implementation of constructive and perturbative local search routines
      Check:   README and gpl.txt
      Remark:  the code is very rudimentary and slow (no speed-ups used)
      Copyright (C) 2004  Thomas Stuetzle
*/

#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <limits.h>
#include <stdlib.h>


#include "global.h"

%%

r  tries        " # number of independent trials"  reqarg
i  inputfile    " f inputfile (SMWC format necessary)" reqarg
l  localsearch  " 0: no ls 1: transpose 2: exchange  3: insert  4: exchange-insert  5: insert-exchange  6: transpose-exchange-insert  7: transpose-insert-exchange " reqarg
c  construction " 0: random starts  1: canonical (1, 2, .. n)  3: WSPT heuristic" reqarg
h  help         " display this help text and exit"             return

%%

void check_out_of_range ( double value, double MIN, double MAX, char *optionName )
/* check whether parameter values are within allowed range */
{
  if ((value<MIN)||(value>MAX)){
    fprintf(stderr,"Error: Option `%s' out of range\n",optionName);
    exit(1);
  }
}

int parse_commandline (int argc, char *argv [])
{
    int i;
    const char *progname;
    struct options options;

    progname = argv [0] != NULL && *(argv [0]) != '\0'
        ? argv [0]
        : "smwc";

    i = parse_options (&options, progname, argc, argv);

    if (i < 1)
    {
        fprintf (stderr, "No options are specified\n");
        fprintf (stderr, "Try `%s --help' for more information.\n",
                 progname);
        exit(1);
    }

    if (options.opt_help)
    {
        printf ("Usage: %s [OPTION]... [ARGUMENT]...\n\n"
                "Options:\n" STR_HELP, progname);
        exit(0);
    }
}

int parse_commandline (int argc, char *argv [])
{
    int i;
    const char *progname;
    struct options options;

    progname = argv [0] != NULL && *(argv [0]) != '\0'
        ? argv [0]
        : "smwc";

    i = parse_options (&options, progname, argc, argv);

    if (i < 1)
    {
        fprintf (stderr, "No options are specified\n");
        fprintf (stderr, "Try `%s --help' for more information.\n",
                 progname);
        exit(1);
    }

    if (options.opt_help)
    {
        printf ("Usage: %s [OPTION]... [ARGUMENT]...\n\n"
                "Options:\n" STR_HELP, progname);
        exit(0);
    }

/*      puts ("\t OPTIONS:"); */


    if ( options.opt_tries ) {
        max_no_tries = atol(options.arg_tries);
/*          fputs ("  -r  --tries ", stdout); */
/*          if (options.arg_tries != NULL) */
/*              printf ("with argument \"%ld\"\n", max_no_tries); */
        check_out_of_range( max_no_tries, 1, 10000, "max_tries (tries)");
    } else {
        max_no_tries = 1;
/*          fprintf(stderr,"\tNote: number or trials is set to default 10\n"); */
    }


    if ( options.opt_inputfile )
    {
        strncpy( name_buf, options.arg_inputfile, strlen( options.arg_inputfile ) );
/*        fputs ("  -i  --inputfile ", stdout);
        if (options.arg_inputfile != NULL)
            printf ("with argument \"%s\"\n", name_buf );
*/
    }


    if ( options.opt_localsearch ) {
        local_search_flag = atol(options.arg_localsearch);
/*        fputs ("  -l  --localsearch ", stdout);
        if (options.arg_localsearch != NULL)
            printf ("with argument \"%ld\"\n", local_search_flag);
*/      
	check_out_of_range( local_search_flag, 0, LONG_MAX, "local_search_flag");
    } else {
        local_search_flag = 3;
/*          fprintf(stderr,"\tNote: local search flag is set to default 3 (insert)\n"); */
    }


    if ( options.opt_construction ) {
        constructive_flag = atol(options.arg_construction);
/*        fputs ("  -c  --construction ", stdout);
        if (options.arg_construction != NULL)
            printf ("with argument \"%ld\"\n", constructive_flag);
*/
        check_out_of_range( constructive_flag, 0, 2, "constructive_flag");
    } else {
        constructive_flag = 2;
/*          fprintf(stderr,"\tNote: constructive search flag is set to default 2 (WSPT)\n"); */
    }


/*      puts ("Non-option arguments:"); */

    while (i < argc)
        printf ("  \"%s\"\n", argv [i++]);

    return 0;
}



