/*

######################################################
###        simple LS algorithms for the SMWC       ###
######################################################

      Version: 1.0
      File:    utilities.c
      Author:  Thomas Stuetzle
      Purpose: implementation of constructive and perturbative local search routines
      Check:   README and gpl.txt
      Remark:  the code is very basic and slow (no speed-ups are implemented)
      Copyright (C) 2004  Thomas Stuetzle
*/

#include <time.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include <assert.h>
#include <limits.h>

#include "global.h"
#include "ls.h"
#include "utilities.h"
#include "parse.h"

long int seed;   /* seed for the random number generator */


void init_program(int argc, char *argv[]) {
/* auxiliary routine for managing program start-up */ 
    setbuf(stdout,NULL);

    local_search_flag = 5;
    constructive_flag = 2; 

    parse_commandline(argc, argv);

    open_input_file(name_buf);

    read_problem_size(input);
    read_deadline(input);
    read_instance(input);
  
    assert(max_no_tries <= 10000);
}


void swap2(float v[], long int c[], long int i, long int j)
/* auxiliary routine for quicksort (called from sort2 below) */
{
    long int  tmp;
    float     tmp2;

    tmp2 = v[i];
    v[i] = v[j];
    v[j] = tmp2;
    tmp = c[i];
    c[i] = c[j];
    c[j] = tmp;
}

void sort2(float v[], long int c[], long int left, long int right)
/* implementation of quicksort (keeps track of original indices in array c */
{
    long int  k, last;

    if (left >= right) 
	return;
    swap2(v, c, left, (left + right)/2);
    last = left;
    for (k=left+1; k <= right; k++)
	if (v[k] > v[left])
	    swap2(v, c, ++last, k);
    swap2(v, c, left, last);
    sort2(v, c, left, last);
    sort2(v, c, last+1, right);
}


double ran01( long *idum ) {
/* returns a pseudo-random number uniformly distributed in [0,1] */

    long k;
    double ans;

    k =(*idum)/IQ;
    *idum = IA * (*idum - k * IQ) - IR * k;
    if (*idum < 0 ) *idum += IM;
    ans = AM * (*idum);
    return ans;
}



void open_input_file(char input_file_name[100]) {

    input = fopen(input_file_name, "r");
    assert(input != NULL);
}



void read_problem_size( FILE *input ) {
/* Read the dimension of the instance */    

    if( fscanf(input, "%ld", &number_jobs) <= 0) {
	fprintf(stderr, "error reading number of jobs\n");
	exit(1);	
    }
    VERBOSE ( printf(" number_jobs = %ld\n",number_jobs); )
}



void read_deadline( FILE *input ) {
/* Read common deadline for the jobs */

    if( fscanf(input, "%ld ", &deadline) <= 0) {
	fprintf(stderr, "error reading deadline value in data file\n");
	exit(1);	
    }
    VERBOSE ( printf(" Best value known so far = %ld\n",deadline); )
}



void read_instance( FILE *input ) {
/* Read the job data of the instance */

    long int 	j;
    
    for ( j = 0 ; j < number_jobs ; j++ ) {
	if( fscanf(input, "%ld", &release_date[j]) <= 0) {
	    fprintf(stderr, "error reading (%ld, %ld) value in data file\n", j, release_date[j]);
	    exit(1);	
	} 
	if( fscanf(input, "%ld", &deadline) <= 0) {
	    fprintf(stderr, "error reading %ld deadline value in data file\n", deadline);
	    exit(1);	
	} 
	if( fscanf(input, "%ld", &proc_time[j]) <= 0) {
	    fprintf(stderr, "error reading (%ld, %ld) processing time in data file\n", j, proc_time[j]);
	    exit(1);	
	} 
	if( fscanf(input, "%ld", &weight[j]) <= 0) {
	    fprintf(stderr, "error reading (%ld, %ld) weight in data file\n", j, weight[j]);
	    exit(1);	
	}
    }

}     

